﻿using System;
using System.Collections.Generic;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using System.Web.UI;
using System.Web.UI.WebControls;
using Microsoft.Office.Word.Server.Conversions;
using System.IO;

namespace Kaboodle.SynchConverter
{
    public partial class Conversion : LayoutsPageBase
    {
        #region Constants

        private const string MSG_INVALID_FILE_TYPE = "You have selected a file which cannot be converted.  Only MS Word formats are currently supported.";
        private const string MSG_NO_FILE = "The specified file cannot be found";

        #endregion

        #region Properties

        //Returns the source library from the ListId web Url parameter value
        private SPList _SourceLibrary = null;
        protected SPList SourceLibrary
        {
            get
            {
                if (_SourceLibrary == null)
                {
                    string ListIdParamValue = Page.Request.Params["ListId"];
                    Guid ListId;
                    if (Guid.TryParse(ListIdParamValue, out ListId))
                    {
                        _SourceLibrary = SPContext.Current.Web.Lists[ListId];
                    }
                }
                return _SourceLibrary;
            }
        }

        //Returns the source document item from the ItemId web Url parameter value
        private SPListItem _SourceDocument = null;
        protected SPListItem SourceDocument
        {
            get
            {
                if (SourceLibrary == null)
                    return null;

                if (_SourceDocument == null)
                {
                    string ItemIdParamValue = Page.Request.Params["ItemId"];
                    int ItemID;
                    if (int.TryParse(ItemIdParamValue, out ItemID))
                    {
                        _SourceDocument = SourceLibrary.GetItemById(ItemID);
                    }
                }

                return _SourceDocument;
            }
        }

        //The format to be returned by the conversion
        private SaveFormat _OutputFormat = SaveFormat.Automatic;
        protected SaveFormat OutputFormat
        {
            get
            {
                if (_OutputFormat == SaveFormat.Automatic)
                {
                    string FormatParamValue = Page.Request["Format"].ToUpper();
                    if (FormatParamValue == "PDF")
                        return SaveFormat.PDF;
                    else if (FormatParamValue == "MHTML")
                        return SaveFormat.MHTML;
                    else if (FormatParamValue == "XPS")
                        return SaveFormat.XPS;
                    else if (FormatParamValue == "RTF")
                        return SaveFormat.RTF;
                }
                return _OutputFormat;
            }
        }

        #endregion

        #region Overrides and initialisation code

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            //Just make sure we have a valid source document to convert
            if ((SourceDocument == null) || (SourceDocument.File == null))
            {
                ReportMessage(MSG_NO_FILE);
            }

            //Bail if the file is not a Word document
            else if (!SourceDocument.File.IsWordFormat())
            {
                ReportMessage(MSG_INVALID_FILE_TYPE);
            }

            //Get here and we have something to convert
            else
            {
                ProcessConversion(SourceDocument.File, OutputFormat);
            }

        }

        #endregion

        #region Methods

        //This method generates the conversion process and then redirects to browser to the converted bytes
        private void ProcessConversion(SPFile File, SaveFormat Format)
        {
            try
            {
                //Get the results of the conversion
                byte[] ConverionBytes = File.ConvertSynchronously(Format);

                //A successful conversion serves the resulting bytes straight back to the user's browser through the response and mime type
                if (ConverionBytes != null)
                {
                    Response.ContentType = OutputFormat.GetMineType();
                    Response.OutputStream.Write(ConverionBytes, 0, ConverionBytes.Length);
                    Response.Flush();
                    Response.Close();
                }
            }
            catch (Exception ex)
            {
                ReportMessage(ex.ToString());
            }
        }

        //Little helper method to add a message to the application page if need be
        private void ReportMessage(string Message)
        {
            labMsg.Text = Message;
        }

        #endregion
    }
}
